
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_CONTENT_DATABASE_H
#define PG_CONTENT_DATABASE_H

#include "xstd/String.h"
#include "xstd/Array.h"

class ostream;
class OBStream;
class IBStream;
class WrBuf;
class InjectIter;


class CdbEntryPrnOpt {
	public:
		CdbEntryPrnOpt();

	public:
		WrBuf *buf;
		InjectIter *injector;
		Size sizeMax;
};

class CdbEntry {
	public:
		virtual ~CdbEntry() {}
		virtual int type() const = 0;

		virtual void image(const String &image) = 0;
		virtual IBStream &load(IBStream &il) = 0;
		virtual OBStream &store(OBStream &ol) const = 0;
		virtual ostream &print(ostream &os) const = 0;
		virtual void print(CdbEntryPrnOpt &opt) const;
};


// contains compiled information about content of Web objects
// can be used to simulate realistic content

class ContentDbase {
	public:
		static CdbEntry *CreateEntry(int type);

	public:
		ContentDbase();
		~ContentDbase();

		const String &name() const { return theName; }
		int count() const; // number of entries
		const CdbEntry *entry(int pos) const { return theEntries[pos]; }

		void add(CdbEntry *e);

		void load(IBStream &is);
		void store(OBStream &os) const;

		ostream &print(ostream &os) const;

	protected:
		Array<CdbEntry*> theEntries;

		mutable String theName; // copied from the last BStream
};


#endif
