
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "xstd/h/strstream.h"
#include "xstd/Socket.h"

#include "base/BStream.h"
#include "runtime/ErrorMgr.h"
#include "runtime/polyErrors.h"
#include "dns/DnsQuery.h"

static String DnsQueryStrName = "dnsquerystr";


bool DnsQuery::sendTo(const NetAddr &ns, Socket &s) {
	char buf[512];
	ostrstream ostr(buf, sizeof(buf));
	OBStream os;
	os.configure(&ostr, DnsQueryStrName);
	
	Assert(dumpAQuery(os));
	Assert(os.offset() <= SizeOf(buf));

	return s.sendTo(buf, os.offset(), ns) == os.offset();
}

bool DnsQuery::dumpAQuery(OBStream &os) const {

	/* header section */

	os << id(); // XXX: check that we dump as short

	// XXX: check that we dump as short
	const Field flags = flagQuery | flagAQuery;
	os << flags;

	os << (Field)1; // QDCOUNT
	os << (Field)0; // ANCOUNT
	os << (Field)0; // NSCOUNT
	os << (Field)0; // ARCOUNT

	/* question section */

	dumpName(os, theQueryAddr);
	os << (Field)typeA;
	os << (Field)classIn;

	return (bool)os;
}

bool DnsQuery::dumpName(OBStream &os, const NetAddr &addr) const {
	// <label-length> <label>
	for (StrIter i(addr.addrA(), '.'); i; ++i) {
		const int len = i.tokLen();
		Assert(0 < len && len < 64);
		os << (char)len;
		os.put(i.tokBeg(), len);
	}
	os << (char)0;
	return (bool)os;
}
