
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl/pgl.h"

#include "xstd/h/math.h"
#include "xstd/h/iostream.h"
#include "xstd/h/strstream.h"

#include "xstd/String.h"
#include "pgl/PglRec.h"
#include "pgl/PglIntSym.h"
#include "pgl/PglNetAddrSym.h"
#include "pgl/PglArraySym.h"
//#include "pgl/PglNetAddrRange.h"
#include "pgl/BenchSideSym.h"
#include "pgl/BenchSym.h"


BenchSideSym::BenchSideSym(): RecSym("BenchSide", new PglRec), theBench(0) {
	theRec->bAdd("addr[]",  "hosts", 0);
	theRec->bAdd("rate", "max_host_load", 0);
	theRec->bAdd("rate", "max_agent_load", 0);
	theRec->bAdd("addr", "addr_mask", 0);
}

BenchSideSym::BenchSideSym(const String &aType, PglRec *aRec): RecSym(aType, aRec), theBench(0) {
}

bool BenchSideSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "BenchSide";
}

SynSym *BenchSideSym::dupe(const String &type) const {
	if (isA(type))
		return new BenchSideSym(this->type(), theRec->clone());
	return RecSym::dupe(type);
}


bool BenchSideSym::hosts(Array<NetAddr*> &addrs) const {
	if (ArraySym *a = getArraySym("hosts")) {
		for (int i = 0; i < a->count(); ++i) {
			const NetAddr &addr = ((const NetAddrSym&)(*a)[i]->cast("addr")).val();
			addrs.append(new NetAddr(addr));
		}
		return true;
	}
	return false;
}

ArraySym *BenchSideSym::hostsSym() const {
	return getArraySym("hosts");
}

NetAddr BenchSideSym::addrMask() const {
	return getNetAddr("addr_mask");
}

NetAddrSym* BenchSideSym::addrMaskSym() const {
	SynSymTblItem *mi;
	Assert(theRec->find("addr_mask", mi));
	if (!mi->sym())
		return 0;
	return &(NetAddrSym&)mi->sym()->cast("addr");
}

bool BenchSideSym::maxHostLoad(double &load) const {
	return getRate("max_host_load", load);
}

bool BenchSideSym::maxAgentLoad(double &load) const {
	return getRate("max_agent_load", load);
}
