
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PGL_ARRAY_SYMBOL_H
#define PGL_ARRAY_SYMBOL_H

#include "pgl/PglListSym.h"

class RndDistr;
class String;

// list of symbols of a specified type
class ArraySym: public ListSym {
	public:
		ArraySym(const String &anItemType);

		virtual bool isA(const String &type) const;

		const String &itemType() const { return theItemType; }

		void addProb(const SynSym &s, double prob);

		// rnd distr of array idxes that matches item probabilities
		RndDistr *makeSelector(const String &name);
		void copyProbs(Array<double> &res) const;
		bool probsVary() const;

		virtual ostream &print(ostream &os, const String &pfx = 0) const;

	protected:
		virtual SynSym *dupe(const String &dType) const;
		virtual ArraySym *cloneType(const String &type) const;
		virtual void addFailure(const SynSym &s);
		virtual bool cadd(const SynSym &s);

		void prob(int idx, double p);
		void syncProbs();
		void makeProbs();
		void makeProbs(Array<double> &res) const;
		void checkProbs() const;
		double statProbs() const;

	protected:
		String theItemType;
		Array<double> theProbs; // member selection probabilities

		mutable int theUndefProbCnt; // items with no probs (cached)
		mutable bool warnedBadProbs;
};


/* template to transform array symbol into arrays of symbols/items
 * the only reason to use a template is to avoid a[i] casting
 * think: maybe the array symbol should be a template instead??
 */

template <class Item>
int ArraySymExportT(const ArraySym &as, const char *typeName, 
	Array<Item*> &arr, Array<double> *probs = 0) {

	arr.stretch(as.count());
	if (probs)
		as.copyProbs(*probs);
	for (int i = 0; i < as.count(); ++i) {
		Item *item = &((Item&)as[i]->cast(typeName));
		arr.append(item);
	}
	return arr.count();
}

/* macro for template-challanged compilers */
#define ArraySymExportM(Item, as, typeName, arr) {         \
	                                                       \
	(arr).stretch((as).count());                           \
	for (int i = 0; i < (as).count(); ++i) {               \
		Item *item = &((Item&)(as)[i]->cast(typeName));    \
		(arr).append(item);                                \
	}                                                      \
}                                                          

/* macro for template-challanged compilers */
#define ArraySymExportMP(Item, as, typeName, arr, probs) { \
	                                                       \
	(arr).stretch((as).count());                           \
	if (probs)                                             \
		(probs)->stretch((as).count());                    \
	for (int i = 0; i < (as).count(); ++i) {               \
		Item *item = &((Item&)(as)[i]->cast(typeName));  \
		(arr).append(item);                                \
		if (probs)                                         \
			(probs)->append((as).prob(i));                 \
	}                                                     \
}                                                          

#endif
