
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl/pgl.h"

#include "pgl/PglRec.h"
#include "pgl/PglRecSym.h"


PglRec::PglRec() {
}

PglRec::~PglRec() {
}

SynSymTblItem **PglRec::valp(const String &key) {
	// we are responsible for the first component of a dot-name only
	String ancor = key;
	String tail = 0;
	if (const char *p = key.chr('.')) {
		ancor = key(0, p-key);
		tail = p+1;
	}

	// lookup ancor
	if (SynSymTblItem **ip = SynSymTbl::valp(ancor)) {
		SynSymTblItem *i = *ip;
		Assert(i);

		// check if we need to go further
		if (tail) {
			if (i->sym() && i->sym()->isA("Rec")) {
				RecSym &s = (RecSym &) i->sym()->cast("Rec");
				return s.rec()->valp(tail);
			}
			// would be nice to complain here, but we lack key's location
			return 0;
		}
		return ip;
	}
	return 0;
}

PglRec *PglRec::clone() const {
	PglRec *rec = new PglRec;

	for (int i = 0; i < count(); ++i) {
		if (valAt(i)) {
			SynSymTblItem *clone = new SynSymTblItem(*valAt(i));

			if (SynSym *s = valAt(i)->sym())
				clone->sym(s->clone(s->type()));

			rec->add(keyAt(i), clone);
		}
	}
	return rec;
}

SynSymTblItem *PglRec::bAdd(const String &type, const String &name, SynSym *s) {
	SynSymTblItem *member = new SynSymTblItem(type, name);
	member->sym(s);
	add(member->name(), member);
	return member;
}
