
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "xstd/BigSize.h"
#include "pgl/PglRec.h"
#include "pgl/PglIntSym.h"
#include "pgl/PglNumSym.h"
#include "pgl/PglTimeSym.h"
#include "pgl/PglSizeSym.h"
#include "pgl/RptmstatSym.h"


RptmstatSym::RptmstatSym(): RecSym("Rptmstat", new PglRec) {
	theRec->bAdd("time", "sample_dur", 0);
	theRec->bAdd("float", "load_delta", 0);
	theRec->bAdd("time", "rptm_min", 0);
	theRec->bAdd("time", "rptm_max", 0);
}

RptmstatSym::RptmstatSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool RptmstatSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Rptmstat";
}

SynSym *RptmstatSym::dupe(const String &type) const {
	if (isA(type))
		return new RptmstatSym(this->type(), theRec->clone());
	return RecSym::dupe(type);
}

bool RptmstatSym::configured() const {
	double d = -1;
	return sampleDur() >= 0 || loadDelta(d) || 
		rptmMin() >= 0 || rptmMax() >= 0;
}

Time RptmstatSym::sampleDur() const {
	return getTime("sample_dur");
}

bool RptmstatSym::loadDelta(double &loadDelta) const {
	return getDouble("load_delta", loadDelta);
}

Time RptmstatSym::rptmMin() const {
	return getTime("rptm_min");
}

Time RptmstatSym::rptmMax() const {
	return getTime("rptm_max");
}
