
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_AGENT_CLASS_H
#define PG_AGENT_CLASS_H

#include "xstd/String.h"
#include "xstd/NetAddr.h"
#include "base/UniqId.h"

class SockOpt;
class AgentSym;
class RndDistr;
class Cache;

// common stuff for servers and clients
class Agent {
	public:
		Agent();
		virtual ~Agent();

		void cache(Cache *aCache);

		const UniqId &id() const { return theId; }
		int seqvId() const { return theSeqvId; }
		const NetAddr &host() const { return theHost; }
		Cache *cache() { return theCache; }

		virtual void start() = 0;

		virtual int logCat() const = 0;

	protected:
		void configure(const AgentSym *cfg, const NetAddr &aHost, SockOpt &opt);

		virtual void describe(ostream &os) const;

	protected:
		UniqId theId;            // unique agent id (random struct)
		int theSeqvId;           // unique agent id (sequential int)

		String theKind;          // user specified label
		NetAddr theHost;         // the [host:]port we live on
		RndDistr *theThinkDistr; // think time for sactions

	private:
		static int TheLastSeqvId;
		Cache *theCache;         // shared cache for proxy agents
};

#endif
