
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef FARM_H
#define FARM_H

#include "xstd/Array.h"

// a collection of Items that configures and recycles its members
// current recycling order is LIFO
// note <Item> -> <Item*> migration
template <class Item>
class Farm: public Array<Item*> {
	public:
		Farm(): theOutLevel(0) {}
		virtual ~Farm() {}// g++ bug: { while (!empty()) destroy(get()); }

		void limit(int limit) { stretch(limit); }
		void populate(int count) { while (count-- > 0) put(gen()); }

		int inLevel() const { return count(); }      // stored
		int outLevel() const { return theOutLevel; } // out there

		Item *getDirty() { theOutLevel++; return empty() ? gen() : pop(); }
		Item *getClean() { return init(getDirty()); }
		Item *get() { return getClean(); }

		void put(Item *i) { theOutLevel--; full() ? overflow(i) : push(i); }

	protected:
		virtual Item *gen() = 0;
		virtual Item *init(Item *i) const { return i; }
		virtual void destroy(Item *i) = 0;

		virtual void overflow(Item *i) { destroy(i); }

	protected:
		int theOutLevel;    // items out there
};

// Fixed capacity Farm for "objects" (new/reset/delete)
template <class Item>
class ObjFarm: public Farm<Item> {
	public:
		ObjFarm() {}
		virtual ~ObjFarm() { while (!empty()) destroy(get()); }// g++ bug: {}

	protected:
		virtual Item *gen() { return new Item(); }
		virtual Item *init(Item *i) const { i->reset(); return i; }
		virtual void destroy(Item *i) { delete i; }
};

#endif
