
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef X_NET_ADDR_H
#define X_NET_ADDR_H

#include "xstd/h/netinet.h"
#include "xstd/String.h"

inline bool operator ==(in_addr a, in_addr b) { return a.s_addr == b.s_addr; }
inline bool operator !=(in_addr a, in_addr b) { return a.s_addr != b.s_addr; }

// inet_addr wrapper which also stores port and
// does _lazy_ ntoa and aton manipulations
class NetAddr {
	public:
		static NetAddr MakeAddr(unsigned long net, unsigned long lna, int port = -1);
		static in_addr NetMask(int subnet);

	public:
		NetAddr(): thePort(-1) { theAddrN.s_addr = INADDR_NONE; }
		NetAddr(const NetAddr &na): theAddrA(na.theAddrA), theAddrN(na.theAddrN), thePort(na.thePort) {}
		NetAddr(const String &addr, int aPort): theAddrA(addr), thePort(aPort) { theAddrN.s_addr = INADDR_NONE; }
		NetAddr(const in_addr &addr, int aPort): theAddrN(addr), thePort(aPort) {}
		NetAddr(const sockaddr_in &a): theAddrN(a.sin_addr), thePort(a.sin_port) {}

		operator void*() const { return (theAddrA || theAddrN.s_addr != INADDR_NONE || thePort >= 0) ? (void*)-1 : 0; }
		bool operator == (const NetAddr &addr) const;
		bool operator != (const NetAddr &addr) const { return !(*this == addr); }
		bool sameButPort(const NetAddr &addr) const;

		const String &addrA() const { if (!theAddrA) syncA(); return theAddrA; }
		const in_addr &addrN() const { if (theAddrN.s_addr == INADDR_NONE) syncN(); return theAddrN; }
		unsigned long lna() const;
		unsigned long net() const;
		int port() const { return thePort; }

		void addr(const String &addr) { theAddrA = addr; theAddrN.s_addr = INADDR_NONE; }
		void addr(in_addr addr) { theAddrN = addr; theAddrA = 0; }
		void port(int aPort) { thePort = aPort; }

		ostream &print(ostream &os) const;

		bool isDomainName() const;

	protected:
		void syncA() const; // theAddrA <- theAddrN
		void syncN() const; // theAddrN <- theAddrA

	protected:
		mutable String theAddrA;  // FQDN or IP; set if not null
		mutable in_addr theAddrN; // IP if available; set if >= 0;
		int thePort;              // set if >= 0;
};

inline ostream &operator <<(ostream &os, const NetAddr &addr) { return addr.print(os); }

#endif
